from typing import Dict, Any, Optional, List
from dataclasses import dataclass
from enum import Enum

class PolicyDecisionResult(str, Enum):
    """Policy decision result enumeration"""
    ALLOW = "allow"
    DENY = "deny"

@dataclass
class RegisterAgentOptions:
    """Options for registering a new agent"""
    agent_name: str
    capabilities: List[str]
    developer_did: Optional[str] = None

@dataclass
class AuthAssertionParams:
    """Parameters for agent authentication assertion"""
    agent_did: str
    vc: str  # Verifiable Credential for this agent
    audience: str  # e.g. https://api.identitymachines.com
    developer_did: Optional[str] = None

@dataclass
class GetAuthTokenOptions:
    """Options for getting authentication token"""
    agent_did: str
    vc: str
    audience: str
    developer_did: Optional[str] = None

@dataclass
class PolicyInput:
    """Input parameters for policy decision"""
    agent_did: str  # agent DID (subject)
    policy_id: str  # specific policy ID to use
    token: str  # short-lived access token (JWT)
    action: str  # e.g. "query"
    resource: str  # e.g. "db://finance/tx"
    context: Optional[Dict[str, Any]] = None  # optional: amount, ticker, etc.

@dataclass
class PolicyDecision:
    """Result of a policy decision"""
    allow: bool
    evaluation: Optional[List[Any]] = None
    reason: Optional[str] = None

@dataclass
class BuildAgentPayloadOptions:
    """Options for building agent payload"""
    agent_name: str
    capabilities: List[str]
    developer_did: str  # DID identifying the agent's owner/issuer

@dataclass
class AgentPayload:
    """Structure sent to Iron Book API and returned to caller"""
    agent_did: str  # did:web:...
    developer_did: str  # did:web:...
    vc: str  # detached JWS VC (compact)
    public_jwk: Dict[str, Any]  # to persist in agent registry for auth token verification
    private_jwk: Dict[str, Any]  # returned for caller to securely store

@dataclass
class UploadPolicyOptions:
    """Options for uploading a policy"""
    config_type: str
    policy_content: str
    metadata: Any
    developer_did: Optional[str] = None

@dataclass
class UpdateAgentOptions:
    """Options for updating an agent"""
    description: Optional[str] = None
    status: Optional[str] = None  # 'active' or 'inactive'

@dataclass
class UpdateAgentResponse:
    """Response from updating an agent"""
    agent_did: str
    developer_did: str
    updated: List[str]  # List of updated field names


# ===============================
# Agent retrieval/listing types
# ===============================

@dataclass
class AgentResponse:
    """Agent record returned by the API (sanitized, no keys)"""
    did: str
    name: Optional[str] = None
    description: Optional[str] = None
    capabilities: Optional[List[str]] = None
    trust_score: Optional[float] = None
    status: Optional[str] = None  # 'active' | 'inactive'
    vc: Optional[str] = None
    org_id: Optional[str] = None
    developer_did: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None


@dataclass
class ListAgentsOptions:
    """Options for listing agents"""
    status: Optional[str] = None  # 'active' | 'inactive'
    capabilities: Optional[List[str]] = None  # All must match
    limit: Optional[int] = None  # 1-100; default 25
    last_key: Optional[str] = None  # Opaque cursor


@dataclass
class ListAgentsResponse:
    """Response from listing agents"""
    items: List[AgentResponse]
    count: int
    last_key: Optional[str] = None

# ===============================
# Policy get/list types
# ===============================

@dataclass
class Policy:
    agent_did: str
    policy_id: Optional[str] = None
    policy_content: Optional[str] = None
    developer_did: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None
    metadata: Optional[Any] = None

@dataclass
class ListPoliciesResponse:
    items: List[Policy]
    count: int
    last_key: Optional[str] = None

@dataclass
class ListPoliciesOptions:
    is_active: Optional[bool] = None
    limit: Optional[int] = None
    last_key: Optional[str] = None

# ===============================
# Audit log listing types
# ===============================

@dataclass
class ListAuditLogsOptions:
    agent_did: Optional[str] = None
    event_type: Optional[str] = None  # REGISTER | AUTH | POLICY | UPDATE
    policy_id: Optional[str] = None
    allow: Optional[bool] = None
    since: Optional[str] = None  # ISO 8601 start
    until: Optional[str] = None  # ISO 8601 end
    limit: Optional[int] = None
    last_key: Optional[str] = None

@dataclass
class ListAuditLogsResponse:
    items: List['AuditLogEntry']
    count: int
    last_key: Optional[str] = None

@dataclass
class AuditLogEntry:
    agent_did: str
    event_type: str  # REGISTER | AUTH | POLICY | UPDATE
    timestamp: str
    trust_score: float
    delta: float
    policy_id: Optional[str] = None
    action: Optional[str] = None
    resource: Optional[str] = None
    allow: Optional[bool] = None
    input: Optional[Dict[str, Any]] = None
    details: Optional[str] = None

# Type aliases for backward compatibility and convenience
RegisterAgentOptions = RegisterAgentOptions
GetAuthTokenOptions = GetAuthTokenOptions
PolicyDecision = PolicyDecision


# ironbook/client.py
import httpx
import json
from typing import Dict, Any, Optional, List
from .types import (
    RegisterAgentOptions, 
    GetAuthTokenOptions, 
    Policy,
    PolicyDecision, 
    PolicyInput,
    UploadPolicyOptions,
    UpdateAgentOptions,
    UpdateAgentResponse,
    AgentResponse,
    ListAgentsOptions,
    ListAgentsResponse,
    ListPoliciesOptions,
    ListPoliciesResponse,
    ListAuditLogsOptions,
    ListAuditLogsResponse,
    AuditLogEntry,
)

class IronBookError(Exception):
    """IronBook SDK error with structured fields."""
    def __init__(self, message: str, *, status: int, code: Optional[str] = None, request_id: Optional[str] = None, details: Optional[Any] = None):
        super().__init__(message)
        self.message = message
        self.status = status
        self.code = code
        self.request_id = request_id
        self.details = details

class IronBookClient:
    """IronBook Trust Service client"""
    
    def __init__(self, api_key: str, base_url: str = "https://api.ironbook.identitymachines.com", timeout: float = 10.0):
        self.api_key = api_key
        self.base_url = base_url
        self.client = httpx.AsyncClient(timeout=httpx.Timeout(timeout))
    
    async def __aenter__(self):
        return self
    
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.client.aclose()
    
    def _get_headers(self) -> Dict[str, str]:
        """Get common headers for API requests"""
        return {
            'Content-Type': 'application/json',
            'x-ironbook-key': self.api_key
        }

    async def _request(self, method: str, path: str, *, json_body: Optional[Dict[str, Any]] = None, headers: Optional[Dict[str, str]] = None) -> Dict[str, Any]:
        url = f"{self.base_url}{path}"
        req_headers = self._get_headers()
        if headers:
            req_headers.update(headers)
        try:
            resp = await self.client.request(method, url, headers=req_headers, json=json_body)
            ct = resp.headers.get('content-type', '')
            is_json = 'json' in ct
            if resp.is_success:
                return resp.json() if is_json else { 'data': resp.text }
            # Build structured error
            body: Any = None
            if is_json:
                try:
                    body = resp.json()
                except json.JSONDecodeError:
                    body = None
            else:
                body = resp.text
            title = body.get('title') if isinstance(body, dict) else None
            fallback = (body.get('error') or body.get('message')) if isinstance(body, dict) else None
            code = body.get('code') or body.get('type') if isinstance(body, dict) else None
            request_id = body.get('requestId') if isinstance(body, dict) else None
            message = title or fallback or f"HTTP {resp.status_code}"
            raise IronBookError(message, status=resp.status_code, code=code, request_id=request_id, details=body)
        except httpx.TimeoutException as e:
            raise IronBookError("Request timed out", status=408, code="REQUEST_TIMEOUT") from e
        except httpx.RequestError as e:
            raise IronBookError(f"Network error: {e}", status=0) from e
        except json.JSONDecodeError as e:
            # successful response but not valid JSON
            raise IronBookError(f"Invalid JSON response: {e}", status=200) from e
    
    async def register_agent(self, opts: RegisterAgentOptions) -> Dict[str, Any]:
        """
        Registers a new agent with the Iron Book Trust Service
        
        Args:
            opts: Registration options including agent name, capabilities, and developer DID
            
        Returns:
            Dict[str, Any]: Response containing vc (Verifiable Credential as a compact-format signed JWT string for this agent), agentDid, and developerDid
            
        Raises:
            IronBookError: If registration fails
        """
        payload = {
            'agentName': opts.agent_name,
            'capabilities': opts.capabilities,
            'developerDID': opts.developer_did
        }
        return await self._request('POST', '/agents/register', json_body=payload)
    
    async def get_auth_token(self, opts: GetAuthTokenOptions) -> Dict[str, Any]:
        """
        Gets a short-lived one-shot JIT access token for the agent to perform an action
        
        Args:
            opts: Authentication options including agent DID, developer DID, VC, and audience
            
        Returns:
            Dict[str, Any]: Response containing access_token and expires_in
            
        Raises:
            IronBookError: If authentication fails
        """
        payload = {
            'agentDid': opts.agent_did,
            'developerDid': opts.developer_did,
            'vc': opts.vc,
            'audience': opts.audience
        }
        return await self._request('POST', '/auth/token', json_body=payload)
    
    async def policy_decision(self, opts: PolicyInput) -> PolicyDecision:
        """
        Gets a policy decision from the Iron Book Trust Service and consumes the one-shot JIT access token
        
        Args:
            opts: Policy decision input including agent DID, token, action, resource, and context
            
        Returns:
            PolicyDecision: Policy decision result with allow/deny and additional details
            
        Raises:
            IronBookError: If policy decision fails
        """
        headers = {'Authorization': f'Bearer {opts.token}'}
        payload = {
            'agentDid': opts.agent_did,  # agent DID
            'policyId': opts.policy_id,   # policy ID
            'action': opts.action,  # e.g. "query"
            'resource': opts.resource,  # e.g. "db://finance/tx"
            'context': opts.context or {}  # optional: amount, ticker, etc.
        }
        data = await self._request('POST', '/policy/decision', json_body=payload, headers=headers)
        return PolicyDecision(
            allow=data.get('allow', False),
            evaluation=data.get('evaluation'),
            reason=data.get('reason')
        )

    async def upload_policy(self, opts: UploadPolicyOptions) -> Dict[str, Any]:
        """
        Uploads a new access control policy to the Iron Book Trust Service
        
        Args:
            opts: Policy upload options including developer DID,
                config type, policy content, metadata, and API key
            
        Returns:
            Response from the policy upload endpoint
            
        Raises:
            IronBookError: If the upload fails
        """
        payload = {
            'developerDid': opts.developer_did,
            'configType': opts.config_type,
            'policyContent': opts.policy_content,
            'metadata': opts.metadata
        }
        return await self._request('POST', '/policies', json_body=payload)

    async def update_agent(self, agent_did: str, opts: UpdateAgentOptions) -> UpdateAgentResponse:
        """
        Updates an agent's description and/or status
        
        Args:
            agent_did: The DID of the agent to update
            opts: Update options including description and/or status
            
        Returns:
            UpdateAgentResponse: Response containing agent DID, developer DID, and updated fields
            
        Raises:
            IronBookError: If update fails
        """
        # Build request body with only provided fields
        request_body: Dict[str, Any] = {}
        if opts.description is not None:
            request_body['description'] = opts.description
        if opts.status is not None:
            request_body['status'] = opts.status
        if not request_body:
            raise IronBookError("At least one of description or status must be provided", status=400, code="VALIDATION_ERROR")

        data = await self._request('PUT', f"/agents/{agent_did}", json_body=request_body)
        return UpdateAgentResponse(
            agent_did=data.get('agentDid'),
            developer_did=data.get('developerDid'),
            updated=data.get('updated', [])
        )

    async def get_agent(self, agent_did: str) -> AgentResponse:
        """Retrieve a single agent by DID"""
        data = await self._request('GET', f"/agents/{agent_did}")
        # Map keys from camelCase to snake_case where present
        return AgentResponse(
            did=data.get('did'),
            name=data.get('name'),
            description=data.get('description'),
            capabilities=data.get('capabilities'),
            trust_score=data.get('trustScore'),
            status=data.get('status'),
            vc=data.get('vc'),
            org_id=data.get('orgID'),
            developer_did=data.get('developerDID'),
            created_at=data.get('createdAt'),
            updated_at=data.get('updatedAt'),
        )

    async def list_agents(self, opts: Optional[ListAgentsOptions] = None) -> ListAgentsResponse:
        """List agents with optional filters and pagination"""
        opts = opts or ListAgentsOptions()
        params: List[str] = []
        if opts.status:
            params.append(f"status={opts.status}")
        if opts.capabilities:
            caps = ','.join(opts.capabilities)
            params.append(f"capabilities={caps}")
        if isinstance(opts.limit, int):
            params.append(f"limit={opts.limit}")
        if opts.last_key:
            params.append(f"lastKey={opts.last_key}")
        qs = ('?' + '&'.join(params)) if params else ''
        data = await self._request('GET', f"/agents{qs}")
        items = [
            AgentResponse(
                did=a.get('did'),
                name=a.get('name'),
                description=a.get('description'),
                capabilities=a.get('capabilities'),
                trust_score=a.get('trustScore'),
                status=a.get('status'),
                vc=a.get('vc'),
                org_id=a.get('orgID'),
                developer_did=a.get('developerDID'),
                created_at=a.get('createdAt'),
                updated_at=a.get('updatedAt'),
            ) for a in data.get('items', [])
        ]
        return ListAgentsResponse(
            items=items,
            count=data.get('count', len(items)),
            last_key=data.get('lastKey')
        )

    async def get_policy(self, policy_id: str) -> Policy:
        """Retrieve a single policy by policyId"""
        data = await self._request('GET', f"/policies/{policy_id}")
        return Policy(
            agent_did=data.get('agentDid'),
            policy_id=data.get('policyId'),
            policy_content=data.get('policyContent'),
            developer_did=data.get('developerDID'),
            created_at=data.get('createdAt'),
        )

    async def list_policies(self, opts: Optional[ListPoliciesOptions] = None) -> ListPoliciesResponse:
        """List policies with optional filters and pagination"""
        opts = opts or ListPoliciesOptions()
        params: List[str] = []
        if isinstance(opts.is_active, bool):
            params.append(f"isActive={'true' if opts.is_active else 'false'}")
        if isinstance(opts.limit, int):
            params.append(f"limit={opts.limit}")
        if opts.last_key:
            params.append(f"lastKey={opts.last_key}")
        qs = ('?' + '&'.join(params)) if params else ''
        data = await self._request('GET', f"/policies{qs}")
        items = [
            Policy(
                agent_did=p.get('agentDid'),
                policy_id=p.get('policyId'),
                policy_content=p.get('policyContent'),
                developer_did=p.get('developerDID'),
                created_at=p.get('createdAt'),
            ) for p in data.get('items', [])
        ]
        return ListPoliciesResponse(
            items=items,
            count=data.get('count', len(items)),
            last_key=data.get('lastKey')
        )

    async def list_audit_logs(self, opts: Optional[ListAuditLogsOptions] = None) -> ListAuditLogsResponse:
        """List audit logs with optional filters and pagination"""
        opts = opts or ListAuditLogsOptions()
        params: List[str] = []
        if opts.agent_did:
            params.append(f"agentDid={opts.agent_did}")
        if opts.event_type:
            params.append(f"eventType={opts.event_type}")
        if opts.policy_id:
            params.append(f"policyId={opts.policy_id}")
        if isinstance(opts.allow, bool):
            params.append(f"allow={'true' if opts.allow else 'false'}")
        if opts.since:
            params.append(f"from={opts.since}")
        if opts.until:
            params.append(f"to={opts.until}")
        if isinstance(opts.limit, int):
            params.append(f"limit={opts.limit}")
        if opts.last_key:
            params.append(f"lastKey={opts.last_key}")
        qs = ('?' + '&'.join(params)) if params else ''
        data = await self._request('GET', f"/audit-logs{qs}")
        items = [
            AuditLogEntry(
                agent_did=a.get('agentDid'),
                event_type=a.get('eventType'),
                timestamp=a.get('timestamp'),
                trust_score=a.get('trustScore'),
                delta=a.get('delta'),
                policy_id=a.get('policyId'),
                action=a.get('action'),
                resource=a.get('resource'),
                allow=a.get('allow'),
                input=a.get('input'),
                details=a.get('details'),
            ) for a in data.get('items', [])
        ]
        return ListAuditLogsResponse(items=items, count=data.get('count', 0), last_key=data.get('lastKey'))

# Convenience functions for backward compatibility and simpler usage
async def register_agent(opts: RegisterAgentOptions, api_key: str, base_url: str = "https://api.ironbook.identitymachines.com") -> Dict[str, Any]:
    """Convenience function for registering an agent"""
    async with IronBookClient(api_key, base_url) as client:
        return await client.register_agent(opts)

async def get_auth_token(opts: GetAuthTokenOptions, api_key: str, base_url: str = "https://api.ironbook.identitymachines.com") -> Dict[str, Any]:
    """Convenience function for getting one-shot JIT authentication token"""
    async with IronBookClient(api_key, base_url) as client:
        return await client.get_auth_token(opts)

async def policy_decision(opts: PolicyInput, api_key: str, base_url: str = "https://api.ironbook.identitymachines.com") -> PolicyDecision:
    """Convenience function for getting policy decision"""
    async with IronBookClient(api_key, base_url) as client:
        return await client.policy_decision(opts)

async def upload_policy(opts: UploadPolicyOptions, api_key: str, base_url: str = "https://api.ironbook.identitymachines.com") -> Dict[str, Any]:
    """Convenience function for uploading a new policy"""
    async with IronBookClient(api_key, base_url) as client:
        return await client.upload_policy(opts)

async def update_agent(agent_did: str, opts: UpdateAgentOptions, api_key: str, base_url: str = "https://api.ironbook.identitymachines.com") -> UpdateAgentResponse:
    """Convenience function for updating an agent"""
    async with IronBookClient(api_key, base_url) as client:
        return await client.update_agent(agent_did, opts)

async def get_agent(agent_did: str, api_key: str, base_url: str = "https://api.ironbook.identitymachines.com") -> AgentResponse:
    """Convenience function for retrieving an agent by DID"""
    async with IronBookClient(api_key, base_url) as client:
        return await client.get_agent(agent_did)

async def list_agents(opts: Optional[ListAgentsOptions], api_key: str, base_url: str = "https://api.ironbook.identitymachines.com") -> ListAgentsResponse:
    """Convenience function for listing agents with optional filters"""
    async with IronBookClient(api_key, base_url) as client:
        return await client.list_agents(opts)

async def get_policy(policy_id: str, api_key: str, base_url: str = "https://api.ironbook.identitymachines.com") -> Dict[str, Any]:
    """Convenience function for retrieving a single policy"""
    async with IronBookClient(api_key, base_url) as client:
        return await client.get_policy(policy_id)

async def list_policies(opts: Optional[ListPoliciesOptions], api_key: str, base_url: str = "https://api.ironbook.identitymachines.com") -> Dict[str, Any]:
    """Convenience function for listing policies"""
    async with IronBookClient(api_key, base_url) as client:
        return await client.list_policies(opts)

async def list_audit_logs(opts: Optional[ListAuditLogsOptions], api_key: str, base_url: str = "https://api.ironbook.identitymachines.com") -> ListAuditLogsResponse:
    """Convenience function for listing audit logs"""
    async with IronBookClient(api_key, base_url) as client:
        return await client.list_audit_logs(opts)


import asyncio
import pytest
from ironbook_sdk.client import (
    register_agent,
    get_auth_token,
    upload_policy,
    policy_decision,
    RegisterAgentOptions,
    GetAuthTokenOptions,
    UploadPolicyOptions,
    PolicyInput,
    IronBookError
)

API_KEY = 'ironbook-api-a4f19d3b72e08c5fa1c367d0bb2c6a9e'
AGENT_NAME = 'python-test-agent'
CAPABILITIES = ['read', 'write', 'goof_around']
DEVELOPER_DID = None  # defaults to 'did:web:identitymachines.com'

POLICY_CONTENT = '''
package policies

default allow = false

allow if {
  input.action == "read"
  input.resource == "user-data"
}

allow if {
  input.action == "goof_around"
}

allow if {
  input.action == "write"
  input.resource == "user-data"
  input.context.user == "admin"
}
'''

@pytest.mark.asyncio
async def test_agent_operations():
    print('\n🤖 Testing Agent Operations...\n')
    try:
        # Test 1: Agent Registration
        print('\n📋 Test: Agent Registration')
        agent_vc = await register_agent(RegisterAgentOptions(
            agent_name=AGENT_NAME,
            capabilities=CAPABILITIES,
            #developer_did=DEVELOPER_DID
        ), api_key=API_KEY)
        print('✅ Agent registration successful:', agent_vc)

        # Test 2: Authentication Token Generation
        print('\n🔐 Test: Authentication Token Generation')
        auth_token_data = await get_auth_token(GetAuthTokenOptions(
            agent_did=agent_vc['agentDid'],
            vc=agent_vc['vc'],
            audience='https://api.identitymachines.com',
            #developer_did=DEVELOPER_DID
        ), api_key=API_KEY)
        access_token = auth_token_data.get('access_token')
        if not access_token:
            print('❌ Failed to obtain access_token from authentication response.')
            return
        print('✅ Authentication token generated successfully:', access_token)

        # Test 3: Policy Upload
        print('\n🔐 Test: Policy Upload')
        policy_response = await upload_policy(UploadPolicyOptions(
            agent_did=agent_vc['agentDid'],
            config_type='opa',
            policy_content=POLICY_CONTENT,
            metadata={"version": "1.0", "description": "Read/write policy"},
            #developer_did=DEVELOPER_DID
        ), api_key=API_KEY)
        print('✅ Policy creation successful:', policy_response) # returns policy object (incl. policyId to use for decision)

        # Test 4: Policy Decision
        print('\n🔐 Test: Policy Decision')
        decision = await policy_decision(PolicyInput(
            agent_did=agent_vc['agentDid'],
            policy_id=policy_response['policyId'],
            token=access_token,
            action='write',
            resource='user-data',
            context={"user": "admin"}
        ), api_key=API_KEY)
        print('✅ Policy decision successful:', decision)

    except IronBookError as e:
        print('❌ Agent operations failed:', str(e))
    except Exception as e:
        print('❌ Unexpected error:', str(e))
    print('\n🎉 Agent operations test completed!')

async def run_all_tests():
    try:
        await test_agent_operations()
        print('\n✨ All tests completed successfully!')
    except Exception as e:
        print('\n💥 Test suite failed:', str(e))

if __name__ == '__main__':
    asyncio.run(run_all_tests())



import pytest
import httpx
import json
from unittest.mock import AsyncMock, patch
from ironbook_sdk.client import IronBookClient, IronBookError
from ironbook_sdk.types import UpdateAgentOptions, UpdateAgentResponse

@pytest.fixture
def client():
    return IronBookClient("test-api-key", "https://test.example.com")

@pytest.fixture
def mock_response():
    return {
        "agentDid": "did:web:agents.identitymachines.com:testagent",
        "developerDid": "did:web:identitymachines.com",
        "updated": ["description", "status"]
    }

class TestUpdateAgent:
    """Test cases for the update_agent method"""

    @pytest.mark.asyncio
    async def test_update_agent_description_only(self, client, mock_response):
        """Test updating only the description"""
        with patch.object(client.client, 'put') as mock_put:
            mock_put.return_value = AsyncMock(
                is_success=True,
                json=lambda: mock_response
            )

            opts = UpdateAgentOptions(description="Updated description")
            result = await client.update_agent("did:web:agents.identitymachines.com:testagent", opts)

            assert isinstance(result, UpdateAgentResponse)
            assert result.agent_did == "did:web:agents.identitymachines.com:testagent"
            assert result.developer_did == "did:web:identitymachines.com"
            assert result.updated == ["description", "status"]

            # Verify the request
            mock_put.assert_called_once_with(
                "https://test.example.com/agents/did:web:agents.identitymachines.com:testagent",
                headers={
                    'Content-Type': 'application/json',
                    'x-ironbook-key': 'test-api-key'
                },
                json={"description": "Updated description"}
            )

    @pytest.mark.asyncio
    async def test_update_agent_status_only(self, client, mock_response):
        """Test updating only the status"""
        with patch.object(client.client, 'put') as mock_put:
            mock_put.return_value = AsyncMock(
                is_success=True,
                json=lambda: mock_response
            )

            opts = UpdateAgentOptions(status="inactive")
            result = await client.update_agent("did:web:agents.identitymachines.com:testagent", opts)

            assert isinstance(result, UpdateAgentResponse)
            assert result.agent_did == "did:web:agents.identitymachines.com:testagent"

            # Verify the request
            mock_put.assert_called_once_with(
                "https://test.example.com/agents/did:web:agents.identitymachines.com:testagent",
                headers={
                    'Content-Type': 'application/json',
                    'x-ironbook-key': 'test-api-key'
                },
                json={"status": "inactive"}
            )

    @pytest.mark.asyncio
    async def test_update_agent_both_fields(self, client, mock_response):
        """Test updating both description and status"""
        with patch.object(client.client, 'put') as mock_put:
            mock_put.return_value = AsyncMock(
                is_success=True,
                json=lambda: mock_response
            )

            opts = UpdateAgentOptions(
                description="Updated description",
                status="active"
            )
            result = await client.update_agent("did:web:agents.identitymachines.com:testagent", opts)

            assert isinstance(result, UpdateAgentResponse)

            # Verify the request
            mock_put.assert_called_once_with(
                "https://test.example.com/agents/did:web:agents.identitymachines.com:testagent",
                headers={
                    'Content-Type': 'application/json',
                    'x-ironbook-key': 'test-api-key'
                },
                json={
                    "description": "Updated description",
                    "status": "active"
                }
            )

    @pytest.mark.asyncio
    async def test_update_agent_no_fields_provided(self, client):
        """Test that an error is raised when no fields are provided"""
        opts = UpdateAgentOptions()
        
        with pytest.raises(IronBookError, match="At least one of description or status must be provided"):
            await client.update_agent("did:web:agents.identitymachines.com:testagent", opts)

    @pytest.mark.asyncio
    async def test_update_agent_http_error(self, client):
        """Test handling of HTTP errors"""
        with patch.object(client.client, 'put') as mock_put:
            mock_put.return_value = AsyncMock(
                is_success=False,
                status_code=404,
                text="Agent not found"
            )

            opts = UpdateAgentOptions(description="Updated description")
            
            with pytest.raises(IronBookError, match="Agent update error \\(404\\): Agent not found"):
                await client.update_agent("did:web:agents.identitymachines.com:testagent", opts)

    @pytest.mark.asyncio
    async def test_update_agent_network_error(self, client):
        """Test handling of network errors"""
        with patch.object(client.client, 'put') as mock_put:
            mock_put.side_effect = httpx.RequestError("Network error")

            opts = UpdateAgentOptions(description="Updated description")
            
            with pytest.raises(IronBookError, match="Network error during agent update"):
                await client.update_agent("did:web:agents.identitymachines.com:testagent", opts)

    @pytest.mark.asyncio
    async def test_update_agent_invalid_json(self, client):
        """Test handling of invalid JSON responses"""
        with patch.object(client.client, 'put') as mock_put:
            mock_response = AsyncMock()
            mock_response.is_success = True
            # Use a regular Mock for the json method since it's not async
            from unittest.mock import Mock
            mock_response.json = Mock(side_effect=json.JSONDecodeError("Expecting value", "invalid json", 0))
            mock_put.return_value = mock_response

            opts = UpdateAgentOptions(description="Updated description")
            
            with pytest.raises(IronBookError, match="Invalid JSON response during agent update"):
                await client.update_agent("did:web:agents.identitymachines.com:testagent", opts)

    @pytest.mark.asyncio
    async def test_update_agent_url_encoding(self, client, mock_response):
        """Test that agent DID is properly URL encoded"""
        with patch.object(client.client, 'put') as mock_put:
            mock_put.return_value = AsyncMock(
                is_success=True,
                json=lambda: mock_response
            )

            opts = UpdateAgentOptions(description="Updated description")
            agent_did = "did:web:agents.identitymachines.com:test:agent"
            
            await client.update_agent(agent_did, opts)

            # Verify the URL encoding
            mock_put.assert_called_once_with(
                "https://test.example.com/agents/did:web:agents.identitymachines.com:test:agent",
                headers={
                    'Content-Type': 'application/json',
                    'x-ironbook-key': 'test-api-key'
                },
                json={"description": "Updated description"}
            )

class TestUpdateAgentConvenienceFunction:
    """Test cases for the convenience update_agent function"""

    @pytest.mark.asyncio
    async def test_convenience_function(self, mock_response):
        """Test the convenience function works correctly"""
        with patch('ironbook_sdk.client.IronBookClient') as mock_client_class:
            mock_client = AsyncMock()
            mock_client_class.return_value = mock_client
            mock_client.__aenter__.return_value = mock_client
            mock_client.update_agent.return_value = UpdateAgentResponse(
                agent_did="did:web:agents.identitymachines.com:testagent",
                developer_did="did:web:identitymachines.com",
                updated=["description"]
            )

            from ironbook_sdk.client import update_agent
            
            opts = UpdateAgentOptions(description="Updated description")
            result = await update_agent("did:web:agents.identitymachines.com:testagent", opts, "test-api-key")

            assert isinstance(result, UpdateAgentResponse)
            assert result.agent_did == "did:web:agents.identitymachines.com:testagent"
            assert result.updated == ["description"]

            # Verify the client was used correctly
            mock_client_class.assert_called_once_with("test-api-key", "https://dev.identitymachines.com")
            mock_client.update_agent.assert_called_once_with("did:web:agents.identitymachines.com:testagent", opts)
            mock_client.__aenter__.assert_called_once()
            mock_client.__aexit__.assert_called_once() 


from .client import IronBookClient
from .types import (
    RegisterAgentOptions,
    GetAuthTokenOptions,
    PolicyDecision,
    PolicyDecisionResult,
    AuthAssertionParams,
    PolicyInput,
    BuildAgentPayloadOptions,
    AgentPayload,
    UploadPolicyOptions,
    UpdateAgentOptions,
    UpdateAgentResponse,
    AgentResponse,
    ListAgentsOptions,
    ListAgentsResponse,
    Policy,
    ListPoliciesResponse,
    ListPoliciesOptions,
    ListAuditLogsOptions,
    ListAuditLogsResponse,
)

__version__ = "0.2.0"
__all__ = [
    "IronBookClient",
    "RegisterAgentOptions",
    "GetAuthTokenOptions", 
    "PolicyDecision",
    "PolicyDecisionResult",
    "AuthAssertionParams",
    "PolicyInput",
    "BuildAgentPayloadOptions",
    "AgentPayload",
    "UploadPolicyOptions",
    "UpdateAgentOptions",
    "UpdateAgentResponse",
    "AgentResponse",
    "ListAgentsOptions",
    "ListAgentsResponse",
    "Policy",
    "ListPoliciesResponse",
    "ListPoliciesOptions",
    "ListAuditLogsOptions",
    "ListAuditLogsResponse",
]